################################################################################
#                                                                              #
#                                LEGAL  NOTICE:                                #
#                                ==============                                #
# The following code and documentation (hereinafter "the script") is an        #
# example script for demonstration purposes only.                              #
# The script is not a standard commercial product offered by Ixia and have     #
# been developed and is being provided for use only as indicated herein. The   #
# script [and all modifications, enhancements and updates thereto (whether     #
# made by Ixia and/or by the user and/or by a third party)] shall at all times #
# remain the property of Ixia.                                                 #
#                                                                              #
# Ixia does not warrant (i) that the functions contained in the script will    #
# meet the user's requirements or (ii) that the script will be without         #
# omissions or error-free.                                                     #
# THE SCRIPT IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, AND IXIA        #
# DISCLAIMS ALL WARRANTIES, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE,          #
# INCLUDING BUT NOT LIMITED TO ANY WARRANTY OF MERCHANTABILITY AND FITNESS FOR #
# A PARTICULAR PURPOSE OR OF NON-INFRINGEMENT.                                 #
# THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SCRIPT  IS WITH THE #
# USER.                                                                        #
# IN NO EVENT SHALL IXIA BE LIABLE FOR ANY DAMAGES RESULTING FROM OR ARISING   #
# OUT OF THE USE OF, OR THE INABILITY TO USE THE SCRIPT OR ANY PART THEREOF,   #
# INCLUDING BUT NOT LIMITED TO ANY LOST PROFITS, LOST BUSINESS, LOST OR        #
# DAMAGED DATA OR SOFTWARE OR ANY INDIRECT, INCIDENTAL, PUNITIVE OR            #
# CONSEQUENTIAL DAMAGES, EVEN IF IXIA HAS BEEN ADVISED OF THE POSSIBILITY OF   #
# SUCH DAMAGES IN ADVANCE.                                                     #
# Ixia will not be required to provide any software maintenance or support     #
# services of any kind (e.g., any error corrections) in connection with the    #
# script or any part thereof. The user acknowledges that although Ixia may     #
# from time to time and in its sole discretion provide maintenance or support  #
# services for the script, any such services are subject to the warranty and   #
# damages limitations set forth herein and will not obligate Ixia to provide   #
# any additional maintenance or support services.                              #
#                                                                              #
################################################################################

################################################################################
#                                                                              #
# Description:                                                                 #
#    This sample configures 10 eCPRI RE sessions on each of the one ports      #
#    and 10 eCPRI REC on the other port, then adds a traffic Item that uses    #
#    IPv4 endpoints, sends traffic and checks the loss using the statistics    #
#                                                                              #
################################################################################

################################################################################
# Import the IxNet library
################################################################################
import sys, os, time, traceback
import logging
# Import the RestPy module
from ixnetwork_restpy.testplatform.testplatform import TestPlatform
from ixnetwork_restpy.assistants.statistics.statviewassistant import StatViewAssistant
from time import sleep

# Enabling DEBUG mode to find out what is happening in backend.
logging.basicConfig(level=logging.DEBUG)

ixnetwork_ip = 'localhost'
rest_port = '11009'
osPlatform = 'windows'

# Forcefully take port ownership if the portList are owned by other users.
forceTakePortOwnership = True

ixChassisIpList = ['10.39.50.96']
portList = [[ixChassisIpList[0], 10,21], [ixChassisIpList[0], 10, 23]]

# END HARNESS VARS ************************************************************
# For linux and connection_manager only. Set to True to leave the session alive for debugging.
debugMode = False

try:
    testPlatform = TestPlatform(ip_address=ixnetwork_ip, rest_port=rest_port, platform=osPlatform, log_file_name='restpy.log')
    # Console output verbosity: None|request|'request response'
    testPlatform.Trace = 'request_response'
    ################################################################################
    # Connect to IxNet client
    ################################################################################
    session = testPlatform.Sessions.add()

    ################################################################################
    # Cleaning up IxNetwork
    ################################################################################
    print("Cleaning up IxNetwork...")
    ixNetwork = session.Ixnetwork
    ixNetwork.NewConfig()

    ################################################################################
    # Adding ports to configuration
    ################################################################################
    print("Adding ports to configuration")
    # Create vports and name them so you could use .find() to filter vports by the name.
    vport1 = ixNetwork.Vport.add(Name='Port1')
    vport2 = ixNetwork.Vport.add(Name='Port2')

    # Assign ports
    testPorts = []
    vportList = [vport.href for vport in ixNetwork.Vport.find()]
    for port in portList:
        testPorts.append(dict(Arg1=port[0], Arg2=port[1], Arg3=port[2]))

    ixNetwork.AssignPorts(testPorts, [], vportList, forceTakePortOwnership)

    ################################################################################
    # Configuring Topologies and Device Groups
    ################################################################################   
    ixNetwork.info('Creating Topology Group 1')
    topology1 = ixNetwork.Topology.add(Name='Topo1', Ports=vport1)
    deviceGroup1 = topology1.DeviceGroup.add(Name='DG1', Multiplier='1')

    ixNetwork.info('Creating Topology Group 2')
    topology2 = ixNetwork.Topology.add(Name='Topo2', Ports=vport2)
    deviceGroup2 = topology2.DeviceGroup.add(Name='DG2', Multiplier='1')
    
    ################################################################################
    # Configuring Ethernet Endpoints
    ################################################################################ 
    ixNetwork.info('Creating Ethernet 1 in Topology 1')
    ethernet1 = deviceGroup1.Ethernet.add(Name='Ethernet 1')
    ethernet1.Mac.Increment(start_value='00:01:01:01:00:01', step_value='00:00:00:00:00:01')
    ethernet1.EnableVlans.Single(True)

    ixNetwork.info('Configuring vlanID for Ethernet 1')
    vlanObj = ethernet1.Vlan.find()[0].VlanId.Increment(start_value=103, step_value=0)
    
    ixNetwork.info('Creating Ethernet 2 in Topology 1')
    ethernet2 = deviceGroup2.Ethernet.add(Name='Ethernet 2')
    ethernet2.Mac.Increment(start_value='00:01:01:02:00:01', step_value='00:00:00:00:00:01')
    ethernet2.EnableVlans.Single(True)

    ixNetwork.info('Configuring vlanID for Ethernet 2')
    vlanObj = ethernet2.Vlan.find()[0].VlanId.Increment(start_value=103, step_value=0)

    ################################################################################
    # Configuring IPv4 Endpoints
    ################################################################################ 
    ixNetwork.info('Configuring IPv4')
    ip1 = ethernet1.Ipv4.add(Name='IPv4 1')
    ip1.Address.Increment(start_value='1.1.1.1', step_value='0.0.0.1')
    ip1.GatewayIp.Increment(start_value='1.1.1.2', step_value='0.0.0.0')

    ixNetwork.info('Configuring IPv4 2')
    ip2 = ethernet2.Ipv4.add(Name='IPv4 2')
    ip2.Address.Increment(start_value='1.1.1.2', step_value='0.0.0.1')
    ip2.GatewayIp.Increment(start_value='1.1.1.1', step_value='0.0.0.0')   

    ################################################################################
    # Configuring eCPRI REC on Topology 1
    ################################################################################
    ixNetwork.info('Configuring eCPRI REC')
    erec = ip1.ECpriRec.add(Name='Ecpri REC')
    erec.MessageType.add(Type="onewayDelayMeasurement")

    ################################################################################
    # Configuring eCPRI RE on Topology 1
    ################################################################################
    ixNetwork.info('Configuring eCPRI RE')
    ere = ip2.ECpriRe.add(Name='Ecpri RE')
    ere.MessageType.add(Type="remoteMemoryAccess")

    ################################################################################
    # Start all Protocol
    ################################################################################
    ixNetwork.StartAllProtocols(Arg1='sync')
    
    sleep(10)

    ixNetwork.info('Create Traffic Item')
    trafficItem = ixNetwork.Traffic.TrafficItem.add(Name='eCPRI Traffic', BiDirectional=False, TrafficType='ipv4')

    ixNetwork.info('Add endpoint flow group')
    trafficItem.EndpointSet.add(Sources=erec, Destinations=ip2)

    # Note: A Traffic Item could have multiple EndpointSets (Flow groups).
    #       Therefore, ConfigElement is a list.

    ixNetwork.info('Configuring config elements')
    configTrafficElement = trafficItem.ConfigElement.find()[0]
    configTrafficElement.FrameRate.Type = 'framesPerSecond'
    configTrafficElement.FrameRate.Rate = 10
    configTrafficElement.TransmissionControl.FrameCount = 10000
    configTrafficElement.TransmissionControl.Type = 'fixedFrameCount'
    configTrafficElement.FrameRateDistribution.PortDistribution = 'splitRateEvenly'
    configTrafficElement.FrameSize.FixedSize = 128
    trafficItem.Tracking.find()[0].TrackBy = ['ecpriMessageType0']
    ################################################################################
    # Generate, apply and start traffic
    ################################################################################
    trafficItem.Generate()
    sleep(5)
    ixNetwork.Traffic.Apply()
    sleep(5)
    ixNetwork.Traffic.Start()
    sleep(50)
    #if debugMode == False:
        # For linux and connection_manager only
     #   session.remove()

except Exception as errMsg:
    print('\n%s' % traceback.format_exc())

    if debugMode == False and 'session' in locals():
        session.remove()